@@ -1,4 +1,5 @@
 """Sends items to Squirro."""
+import base64
 import copy
 import logging
 import math
@@ -73,6 +74,10 @@ class ItemUploader(object):
     :param pipeline_workflow_name: Pipeline workflow name. Either name or id
         need to be set.
     :param pipeline_workflow_id: Pipeline workflow ID.
+    :param dataloader_options: Dataloader options to store along with a creation of a
+        new data source.
+    :param dataloader_plugin_options: Dataloader plugin options to store with a creation
+        of a new data source.
     :param non_retry_list: List of status codes for which we don't want a
         retry/backoff logic. Defaults to `[200, 202, 401, 403, 400, 404]`.
 
@@ -140,8 +145,10 @@ class ItemUploader(object):
         pipeline_workflow_name=None,
         pipeline_workflow_id=None,
         timeout_secs=None,
+        dataloader_options=None,
+        dataloader_plugin_options=None,
         non_retry_list=[200, 202, 400, 401, 403, 404],
-        **kwargs
+        **kwargs,
     ):
         self.non_retry_list = non_retry_list
         if source_secret:
@@ -253,7 +260,10 @@ class ItemUploader(object):
                 self.project_id,
             )
             source = self._create_source(
-                source_name=source_name, pipeline_workflow_id=workflow["id"]
+                source_name=source_name,
+                pipeline_workflow_id=workflow["id"],
+                dataloader_options=dataloader_options,
+                dataloader_plugin_options=dataloader_plugin_options,
             )
             log.info("New source created with id %r", source["id"])
 
@@ -447,15 +457,34 @@ class ItemUploader(object):
 
         return source
 
-    def _create_source(self, source_name, pipeline_workflow_id):
-        source_config = {
-            "dataloader_options": {
-                "project_id": self.project_id,
-                "plugin_name": "ITEMUPLOADER",
-                "runs_on_client": True,
-            },
-            "dataloader_plugin_options": {},
+    def _create_source(
+        self,
+        source_name,
+        pipeline_workflow_id,
+        dataloader_options=None,
+        dataloader_plugin_options=None,
+    ):
+        default_dataloader_options = {
+            "project_id": self.project_id,
+            "plugin_name": "ITEMUPLOADER",
+            "runs_on_client": True,
         }
+        default_dataloader_plugin_options = {}
+
+        source_config = {}
+
+        if dataloader_options:
+            source_config["dataloader_options"] = dataloader_options
+            source_config["dataloader_options"]["runs_on_client"] = True
+        else:
+            source_config["dataloader_options"] = default_dataloader_options
+
+        if dataloader_plugin_options:
+            source_config["dataloader_plugin_options"] = dataloader_plugin_options
+        else:
+            source_config[
+                "dataloader_plugin_options"
+            ] = default_dataloader_plugin_options
 
         res = self.client.new_source(
             project_id=self.project_id,
@@ -509,43 +538,23 @@ class ItemUploader(object):
         if pipeline_workflow_id is not None:
             params.update({"pipeline_workflow_id": pipeline_workflow_id})
 
-        batch_size = self._get_batch_size(items)
-
-        if not items:
-            log.info("Asked to upload empty item list. Skipping.")
-        num_batches = math.ceil(len(items) / batch_size)
-        for idx in range(0, len(items), batch_size):
-            mdelay = delay
-            for item in items[idx : idx + batch_size]:
-                self._fixup_keywords(item)
-            data = json.dumps(items[idx : idx + batch_size])
-            for i in range(num_retries + 1):
-                if i != 0:
-                    log.warning("Retrying {} of {}".format(i, num_retries))
-                res = self.client._perform_request(
-                    "post", url, params=params, data=data, headers=headers
-                )
-                if res.status_code in self.non_retry_list:
-                    self.client._process_response(res, [202])
-                    break
-                else:
-                    log.warning(
-                        "Failed to upload batch {} of {}, returned "
-                        "status code {}. Sleeping for {} seconds.".format(
-                            idx + 1, num_batches, res.status_code, mdelay
-                        )
-                    )
-                    time.sleep(mdelay)
-                    mdelay *= backoff
+        self._upload(items, url, params, headers, num_retries, delay, backoff)
 
     def upload_rows(
-        self, rows, priority=0, pipeline_workflow_id=None,
+        self,
+        rows,
+        priority=0,
+        pipeline_workflow_id=None,
+        num_retries=10,
+        delay=1,
+        backoff=2,
     ):
         """Sends ``rows`` to Squirro.
 
         :param rows: A list of rows. Row means a single row of data that will eventually
             transformed, enriched, and become a Squirro item. In other words, a row is a
-            single data point/sample of raw data.
+            single data point/sample of raw data, as extracted by a Squirro dataloader
+            plugin.
         :param priority: int, describing the priority of ingestion for the dataset to be
             loaded. Currently only supports a value of `0` or `1`.
             `0` means that the items are loaded in an asynchronous fashion and
@@ -553,6 +562,11 @@ class ItemUploader(object):
         :param pipeline_workflow_id: str, id of an existing pipeline workflow which
             should be used to process the current batch of items. Can only be used with
             parameter `priority` set to 1.
+        :param num_retries: int, Number of retries to make when a service is
+            unavailable.
+        :param delay: int, Initial delay in seconds between retries.
+        :param backoff: int, Backoff multiplier, e.g. value of 2 will double
+            the delay each retry.
         """
         self._validate_upload_parameters(priority, pipeline_workflow_id)
         if not self.config.get("provider_api_url"):
@@ -570,12 +584,21 @@ class ItemUploader(object):
         if pipeline_workflow_id is not None:
             params.update({"pipeline_workflow_id": pipeline_workflow_id})
 
-        data = json.dumps(rows)
+        # in order to dump successfully the rows to json, we need first to transform any
+        # binary data to text (e.g. `file_data` of a PDF file)
+        base64_encoded_columns = set()
+        for row in rows:
+            for column_name, value in row.items():
+                if isinstance(value, (bytes, bytearray)):
+                    row[column_name] = base64.b64encode(value).decode()
+                    base64_encoded_columns.add(column_name)
 
-        res = self.client._perform_request(
-            "post", url, params=params, data=data, headers=headers
-        )
-        self.client._process_response(res)
+        if base64_encoded_columns:
+            # cannot pass it as a param to the endpoint with a list value
+            base64_encoded_columns = ",".join(col for col in base64_encoded_columns)
+            params.update({"base64_encoded_columns": base64_encoded_columns})
+
+        self._upload(rows, url, params, headers, num_retries, delay, backoff)
 
     def _validate_upload_parameters(self, priority, pipeline_workflow_id):
         """Validations for parameters to the `upload` function."""
@@ -632,6 +655,36 @@ class ItemUploader(object):
                     "is not supported".format(key, type(value))
                 )
 
+    def _upload(self, items, url, params, headers, num_retries, delay, backoff):
+        if not items:
+            log.info("Asked to upload empty item list. Skipping.")
+            return
+
+        batch_size = self._get_batch_size(items)
+        num_batches = math.ceil(len(items) / batch_size)
+        for idx in range(0, len(items), batch_size):
+            mdelay = delay
+            for item in items[idx : idx + batch_size]:
+                self._fixup_keywords(item)
+            data = json.dumps(items[idx : idx + batch_size])
+            for i in range(num_retries + 1):
+                if i != 0:
+                    log.warning(f"Retrying {i} of {num_retries}")
+                res = self.client._perform_request(
+                    "post", url, params=params, data=data, headers=headers
+                )
+                if res.status_code in self.non_retry_list:
+                    self.client._process_response(res, [202])
+                    break
+                else:
+                    log.warning(
+                        f"Failed to upload batch {int(idx / batch_size) + 1} of "
+                        f"{num_batches}, returned status code {res.status_code}. "
+                        f"Sleeping for {mdelay} seconds."
+                    )
+                    time.sleep(mdelay)
+                    mdelay *= backoff
+
     # default values for arguments
     DEFAULTS = {
         "source_name": "Upload",
