@@ -7,6 +7,7 @@ from ..util import deprecation
 from .communities import CommunitiesMixin
 from .community_subscription import CommunitySubscriptionsMixin
 from .community_types import CommunityTypesMixin
+from .configuration import ConfigurationMixin
 from .contributingrecords import ContributingRecordsMixin
 from .dashboards import DashboardsMixin
 from .email_templates import EmailTemplatesMixin
@@ -1584,8 +1585,9 @@ class TopicApiBaseMixin(object):
 class TopicApiMixin(
     TopicApiBaseMixin,
     CommunitiesMixin,
-    CommunityTypesMixin,
     CommunitySubscriptionsMixin,
+    CommunityTypesMixin,
+    ConfigurationMixin,
     ContributingRecordsMixin,
     DashboardsMixin,
     EmailTemplatesMixin,
