@@ -294,6 +294,8 @@ class SourcesMixin(object):
         config=None,
         scheduling_options=None,
         pipeline_workflow_id=None,
+        enable_scheduling=None,
+        validate_schema=None,
     ):
         """Modify an existing source.
 
@@ -353,6 +355,10 @@ class SourcesMixin(object):
             data["scheduling_options"] = scheduling_options
         if pipeline_workflow_id is not None:
             data["pipeline_workflow_id"] = pipeline_workflow_id
+        if enable_scheduling is not None:
+            data["enable_scheduling"] = enable_scheduling
+        if validate_schema is not None:
+            data["validate_schema"] = validate_schema
 
         res = self._perform_request("put", url, data=json.dumps(data), headers=headers)
         return self._process_response(res, [200])
